#!/usr/bin/python3
"""
## © University of Southampton IT Innovation Centre, 2018
##
## Copyright in this software belongs to University of Southampton
## IT Innovation Centre of Gamma House, Enterprise Road, 
## Chilworth Science Park, Southampton, SO16 7NS, UK.
##
## This software may not be used, sold, licensed, transferred, copied
## or reproduced in whole or in part in any manner or form or in or
## on any media by any person other than in accordance with the terms
## of the Licence Agreement supplied with the software, or otherwise
## without the prior written consent of the copyright owners.
##
## This software is distributed WITHOUT ANY WARRANTY, without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE, except where stated in the Licence Agreement supplied with
## the software.
##
##      Created By :            Michael Boniface
##      Created Date :          20-03-2018
##      Created for Project :   FLAME
"""

import pytest
import subprocess

def test_write_telegraf_conf():

  # test telegraf monitoring configuration
  TELEGRAF_CONF_DIR="/etc/telegraf"
  LOCATION="DC1"
  SFC_ID="media_service_A"
  SFC_ID_INSTANCE="media_service_A_instance"
  SF_ID="streaming_service"
  SF_ID_INSTANCE="streaming_service_instance"
  IP_ENDPOINT_ID="endpoint"
  SR_ID="service_router"  
  INFLUXDB_URL="http://172.29.236.10"
  DATABASE_NAME="experimentation_database"  

  try:
    # mk telegraf conf directory

    (out, err, code) = run_command('sudo mkdir -p /etc/telegraf')
    assert code == 0, "Failed to create telegraf conf dir : " + str(code) + ", cmd=" + cmd    

    (out, err, code) = run_command('sudo mkdir -p /etc/telegraf/telegraf.d')
    assert code == 0, "Failed to create telegraf include dir : " + str(code) + ", cmd=" + cmd  

    # run write config template  script with no telegraf.d directory
    (out, err, code) = run_command('sudo cp /vagrant/scripts/clmc-agent/telegraf.conf /etc/telegraf/')
    assert code == 0, "Failed to copy telegraf.conf : " + str(code) + ", cmd=" + cmd

    cmd = 'sudo cp /vagrant/scripts/clmc-agent/telegraf_output.conf /etc/telegraf/telegraf.d/'
    (out, err, code) = run_command(cmd)
    assert code == 0, "Failed to copy telegraf_output.conf : " + str(code) + ", cmd=" + cmd

    # run template relacement script with incorrect arguments
    cmd = 'sudo /vagrant/scripts/clmc-agent/configure.sh' 
    (out, err, code) = run_command(cmd)
    assert code == 1, "Failed to return error on incorrect arguments : " + str(code) + ", cmd=" + cmd  

    # run template relacement script with all arguments
    cmd = 'sudo /vagrant/scripts/clmc-agent/configure.sh ' + LOCATION + ' ' + SFC_ID + ' ' + SFC_ID_INSTANCE + ' ' + SF_ID + ' ' + SF_ID_INSTANCE + ' ' + IP_ENDPOINT_ID + ' ' + SR_ID + ' ' + INFLUXDB_URL + ' ' + DATABASE_NAME
    (out, err, code) = run_command(cmd)
    assert code == 0, "Configure command returned error, output=" + str(out) + ", cmd=" + cmd

    # check that replacement was correct in telegraf.conf
    try:        
        TELEGRAF_GENERAL_CONF_FILE = TELEGRAF_CONF_DIR + "/telegraf.conf"
        with open(TELEGRAF_GENERAL_CONF_FILE) as general_conf:
          lines = general_conf.read()          
          assert lines.find(LOCATION), "Cannot find location" 
          assert lines.find(SFC_ID), "Cannot find sfc_id"
          assert lines.find(SFC_ID_INSTANCE), "Cannot find sfc_id_instance"  
          assert lines.find(SF_ID), "Cannot find sfc_id"            
          assert lines.find(SF_ID_INSTANCE), "Cannot find sf_id_instance"
          assert lines.find(IP_ENDPOINT_ID), "Cannot find endpoint"      
          assert lines.find(SR_ID), "Cannot find sr_id"                                
    except FileNotFoundError:
        assert False, "Telegraf general conf file not found, " + TELEGRAF_GENERAL_CONF_FILE

    # check that replacement was correct in telegraf_output.conf
    try:
        TELEGRAF_OUTPUT_CONF_FILE = TELEGRAF_CONF_DIR + "/telegraf.d/telegraf_output.conf"
        with open(TELEGRAF_OUTPUT_CONF_FILE) as output_conf:
          lines = output_conf.read()
          assert lines.find(INFLUXDB_URL), "Cannot find influx_db" 
          assert lines.find(DATABASE_NAME), "Cannot find database"                    
    except FileNotFoundError:
        assert False, "Telegraf output conf file not found, " + TELEGRAF_OUTPUT_CONF_FILE

  finally:
      # clean up telegraf after test
#      run_command("sudo rm -rf /etc/telegraf")
       print("final")
# wrapper for executing commands on the cli, returning (std_out, std_error, process_return_code)
def run_command(cmd):
    """Run a shell command.

    Arguments:
        cmd {string} -- command to run in the shell

    Returns:
        stdout, stderr, exit code -- tuple of the process's stdout, stderr and exit code (0 on success)
    """
    proc = subprocess.Popen([cmd], stdout=subprocess.PIPE, shell=True)
    out, err = proc.communicate()
    return_code = proc.returncode
    return out, err, return_code

